# -*- coding: utf-8 -*-
"""Multichain RPC Client"""
import simplejson as json
import requests
from functools import partial
from decimal import Decimal
from mcrpc.exceptions import RpcError
from mcrpc.base import BaseApiMethods


class RpcClient(BaseApiMethods):

    def __init__(self, host, port, user, pwd, use_ssl=False):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.use_ssl = use_ssl

    @property
    def _url(self):
        url = '{}:{}@{}:{}'.format(self.user, self.pwd, self.host, self.port)
        return 'https://' + url if self.use_ssl else 'http://' + url

    def _call(self, method, *args):
        args = [arg for arg in args if arg is not None]
        payload = {"method": method, "params": args}
        serialized = json.dumps(payload, use_decimal=True)
        response = requests.post(self._url, data=serialized, verify=False)
        data = response.json(parse_float=Decimal)
        if data['error'] is not None:
            raise RpcError(data['error'].get('message'))
        return data['result']

    def __getattr__(self, method):
        return partial(self._call, method)


if __name__ == '__main__':
    c = RpcClient('127.0.0.1', '7010', 'testuser', 'testpassword', False)



# -*- coding: utf-8 -*-
# WARNING: Autogenerated code. Do not edit manually.


class Getblockchaininfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.difficulty = kwargs["difficulty"]
        self.protocol = kwargs["protocol"]
        self.chainname = kwargs["chainname"]
        self.reindex = kwargs["reindex"]
        self.description = kwargs["description"]
        self.verificationprogress = kwargs["verificationprogress"]
        self.blocks = kwargs["blocks"]
        self.chain = kwargs["chain"]
        self.bestblockhash = kwargs["bestblockhash"]
        self.chainwork = kwargs["chainwork"]
        self.headers = kwargs["headers"]
        self.setupblocks = kwargs["setupblocks"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getmempoolinfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.size = kwargs["size"]
        self.bytes = kwargs["bytes"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Gettxoutsetinfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.height = kwargs["height"]
        self.txouts = kwargs["txouts"]
        self.bytes_serialized = kwargs["bytes_serialized"]
        self.total_amount = kwargs["total_amount"]
        self.bestblock = kwargs["bestblock"]
        self.transactions = kwargs["transactions"]
        self.hash_serialized = kwargs["hash_serialized"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getinfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.protocol = kwargs["protocol"]
        self.relayfee = kwargs["relayfee"]
        self.chainname = kwargs["chainname"]
        self.nodeversion = kwargs["nodeversion"]
        self.port = kwargs["port"]
        self.version = kwargs["version"]
        self.burnaddress = kwargs["burnaddress"]
        self.difficulty = kwargs["difficulty"]
        self.miningpaused = kwargs["miningpaused"]
        self.balance = kwargs["balance"]
        self.proxy = kwargs["proxy"]
        self.incomingpaused = kwargs["incomingpaused"]
        self.walletdbversion = kwargs["walletdbversion"]
        self.setupblocks = kwargs["setupblocks"]
        self.paytxfee = kwargs["paytxfee"]
        self.description = kwargs["description"]
        self.connections = kwargs["connections"]
        self.nodeaddress = kwargs["nodeaddress"]
        self.protocolversion = kwargs["protocolversion"]
        self.errors = kwargs["errors"]
        self.testnet = kwargs["testnet"]
        self.timeoffset = kwargs["timeoffset"]
        self.blocks = kwargs["blocks"]
        self.reindex = kwargs["reindex"]
        self.keypoolsize = kwargs["keypoolsize"]
        self.walletversion = kwargs["walletversion"]
        self.keypoololdest = kwargs["keypoololdest"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getruntimeparams:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.gen = kwargs["gen"]
        self.rescan = kwargs["rescan"]
        self.port = kwargs["port"]
        self.lockadminminerounds = kwargs["lockadminminerounds"]
        self.maxshowndata = kwargs["maxshowndata"]
        self.autocombinesuspend = kwargs["autocombinesuspend"]
        self.autocombinemininputs = kwargs["autocombinemininputs"]
        self.autocombineminconf = kwargs["autocombineminconf"]
        self.hideknownopdrops = kwargs["hideknownopdrops"]
        self.bantx = kwargs["bantx"]
        self.autocombinedelay = kwargs["autocombinedelay"]
        self.handshakelocal = kwargs["handshakelocal"]
        self.txindex = kwargs["txindex"]
        self.v1apicompatible = kwargs["v1apicompatible"]
        self.mineemptyrounds = kwargs["mineemptyrounds"]
        self.autosubscribe = kwargs["autosubscribe"]
        self.miningturnover = kwargs["miningturnover"]
        self.miningrequirespeers = kwargs["miningrequirespeers"]
        self.reindex = kwargs["reindex"]
        self.lockblock = kwargs["lockblock"]
        self.autocombinemaxinputs = kwargs["autocombinemaxinputs"]
        self.genproclimit = kwargs["genproclimit"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getmininginfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.currentblocksize = kwargs["currentblocksize"]
        self.currentblocktx = kwargs["currentblocktx"]
        self.errors = kwargs["errors"]
        self.testnet = kwargs["testnet"]
        self.pooledtx = kwargs["pooledtx"]
        self.generate = kwargs["generate"]
        self.difficulty = kwargs["difficulty"]
        self.networkhashps = kwargs["networkhashps"]
        self.hashespersec = kwargs["hashespersec"]
        self.blocks = kwargs["blocks"]
        self.chain = kwargs["chain"]
        self.genproclimit = kwargs["genproclimit"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getnettotals:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.timemillis = kwargs["timemillis"]
        self.totalbytessent = kwargs["totalbytessent"]
        self.totalbytesrecv = kwargs["totalbytesrecv"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getnetworkinfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.networks = kwargs["networks"]
        self.connections = kwargs["connections"]
        self.relayfee = kwargs["relayfee"]
        self.timeoffset = kwargs["timeoffset"]
        self.subversion = kwargs["subversion"]
        self.version = kwargs["version"]
        self.protocolversion = kwargs["protocolversion"]
        self.localservices = kwargs["localservices"]
        self.localaddresses = kwargs["localaddresses"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False


class Getwalletinfo:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.txcount = kwargs["txcount"]
        self.balance = kwargs["balance"]
        self.utxocount = kwargs["utxocount"]
        self.keypoolsize = kwargs["keypoolsize"]
        self.walletdbversion = kwargs["walletdbversion"]
        self.walletversion = kwargs["walletversion"]
        self.keypoololdest = kwargs["keypoololdest"]

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False




# -*- coding: utf-8 -*-
import os
from string import Template
from textwrap import indent
from mcrpc.exceptions import RpcError
from mcrpc.client import RpcClient


HERE = os.path.dirname(__file__)


resp_tpl = """
class {classname}:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        {setters}

    def as_dict(self):
        return self._kwargs

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._kwargs == other._kwargs
        else:
            return False\n
"""


def get_stub_template():
    filepath = os.path.join(HERE, 'client.tpl')
    with open(filepath) as f:
        return Template(f.read())


def get_api_methods(client):
    text = client.help()
    methods = []
    for line in text.splitlines():
        if not line or line.startswith('='):
            continue
        else:
            meth_name = line.split()[0]
            meth_sig = ' '.join(line.split()[1:])
            methods.append((meth_name, meth_sig))
    return methods


def build_response_class(method, resp):
    result = ''
    if not isinstance(resp, dict):
        return result

    setters = []
    for field in resp.keys():
        setter = 'self.{} = kwargs["{}"]'.format(field, field)
        setters.append(setter)
    setters = '\n        '.join(setters)
    return resp_tpl.format(
        classname=method.title(), setters=setters
    )


def build_code(client):
    """Build a best effort base class"""

    base_head = (
        '# -*- coding: utf-8 -*-\n'
        '# WARNING: Autogenerated code. Do not edit manually.\n'
        'from mcrpc.responses import *\n\n\n'
        'class BaseApiMethods:\n\n'
    )
    base_tpl = Template(
        '    def $method(self$sig):\n'
        '        return self._call("$method"$call)\n\n'
    )
    base_tpl_obj = Template(
        '    def $method(self$sig):\n'
        '        data = self._call("$method"$call)\n'
        '        return $resp_class(**data)\n\n'
    )
    base_out = base_head

    stub_tpl = Template(
        '    def $method(self$sig)$rtype:\n'
        '        """\n'
        '$help\n'
        '        """\n\n'
    )
    stub_out = ''

    resp_out = (
        '# -*- coding: utf-8 -*-\n'
        '# WARNING: Autogenerated code. Do not edit manually.\n\n'
    )

    for method, sig in get_api_methods(client):
        help_ = indent(client._call('help', method), 8 * ' ')
        if not sig:
            # We can reliably build methods, stubs and response types/objects
            rtype = ''
            is_obj_resp = False
            if method != 'stop':
                try:
                    resp = client._call(method)
                    if isinstance(resp, dict):
                        resp_out += build_response_class(method, resp)
                        rtype = ' -> ' + method.title()
                        is_obj_resp = True
                    else:
                        rtype = resp.__class__.__name__
                        rtype = ' -> ' + rtype if rtype != 'NoneType' else ' -> None'
                except RpcError:
                    pass
            stub_out += stub_tpl.substitute(
                method=method, sig='', rtype=rtype, help=help_
            )
            if is_obj_resp:
                base_out += base_tpl_obj.substitute(
                    method=method, sig='', call='', resp_class=method.title()
            )
            else:
                base_out += base_tpl.substitute(
                    method=method, sig='', call=''
                )
        else:

            sig_clean = sig\
                .replace('( ', '(')\
                .replace(' )', ')')\
                .replace('"', '')\
                .replace('|', '_or_')\
                .replace('-', '_')\
                .replace('(s)', 's')\
                .replace('(es)', 'es')\
                .replace('from ', 'from_')\
                .replace('hex ', 'hex_')\
                .replace('open ', 'open_')

            is_clean = not any(c in sig_clean for c in '{')

            if is_clean:
                # we can build base class and stub signatures
                args_kwargs = sig_clean.split('(')
                args = args_kwargs[0].split()
                kwargs = args_kwargs[1].strip(')').split() if len(args_kwargs) > 1 else []
                call = ', '.join(args + kwargs)
                sig_kwargs = ['{}=None'.format(kw) for kw in kwargs]
                sig = ', '.join(args + sig_kwargs)

                base_out += base_tpl.substitute(
                    method=method, sig=', ' + sig, call=', ' + call
                )
                stub_out += stub_tpl.substitute(
                    method=method, sig=', ' + sig, rtype='', help=help_
                )
                print(method, '->', sig_clean, '->', args, '->', kwargs)
            else:
                # we can build basic help stubs
                stub_out += stub_tpl.substitute(
                    method=method, sig=', *args', rtype='', help=help_
                )

    base_out_path = os.path.join(HERE, 'base.py')
    with open(base_out_path, 'w') as outf:
        outf.write(base_out)

    stub_out_path = os.path.join(HERE, 'client.pyi')
    stub_out = get_stub_template().substitute(methods=stub_out)
    with open(stub_out_path, 'w') as outf:
        outf.write(stub_out)

    resp_out_path = os.path.join(HERE, 'responses.py')
    with open(resp_out_path, 'w') as outf:
        outf.write(resp_out)


def generate_code():
    client = RpcClient('127.0.0.1', '7010', 'testuser', 'testpassword', False)
    build_code(client)


if __name__ == '__main__':
    generate_code()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='mcrpc',
    version='1.0.1',
    description="mcrpc: MultiChain RPC Library",
    long_description=readme(),
    author="Titusz Pan",
    author_email='tp@py7.de',
    url='https://github.com/coblo/mcrpc',
    packages=find_packages(include=['mcrpc']),
    include_package_data=True,
    install_requires=['requests', 'simplejson'],
    license="GNU General Public License v3",
    zip_safe=False,
    keywords="mcrpc, multichain, rpc, client, api, library",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
    ],
    test_suite='tests',
)
